var connection = {
    url: "jdbc:oracle:thin:@//<>BIRT_ORA_HOST<>:<>BIRT_ORA_PORT<>/<>BIRT_ORA_DBNAME<>",
    userName: "<>BIRT_ORA_USER<>",
    password: "<>BIRT_ORA_PASSWORD<>",
    useWallet: <>BIRT_ORA_WALLET_BOOL<>,
    isSaaSTek: <>TEK_SAAS_BOOL<>,
    vpdUserNamePrefix: "<>BIRT_ORA_WALLET_PREFIX<>",
    walletPrefix: "<>BIRT_ORA_WALLET_PREFIX<>",
    driverClass: "oracle.jdbc.driver.OracleDriver"
}

/**
  * @desc - retorna um objeto das informações utilizadas para acessar a base do relatório
  * @return object
  */
function getConnection() {
    return connection;
}

/**
  * @desc - retornar um parâmetro que está sendo enviado ao relatório.
  * @param string parameterName - nome do parâmetro desejado
  * @return string
  */
function getParameter(parameterName) {
    return reportContext.getParameterValue(parameterName) || '';
}

/**
  * @desc - retorna o driver url para acesso à base de dados
  * @return string
  */
function getUrl()  {
    var connection = getConnection();
    url = connection.url;

    if (connection.useWallet) {
        var nrorg = "";
        if (connection.isSaaSTek) {
            nrorg = getParameter('NRORG') || getParameter('P_NRORG');
        }
        url = 'jdbc:oracle:thin:/@'+ connection.walletPrefix + nrorg;
    }

    return url;
}

/**
  * @desc - retorna o usuário de acesso à base de dados
  * @return string
  */
function getUserName() {
    var connection = getConnection();
    var nrorg = getParameter('NRORG') || getParameter('P_NRORG');

    if (connection.useWallet) {
        return null;
    }
    
    if (connection.isSaaSTek) {
        return connection.vpdUserNamePrefix + nrorg;
    }

    return connection.userName;
}

/**
  * @desc - retorna o senha de acesso à base de dados
  * @return string
  */
function getPassword() {
    var connection = getConnection();
    var password = connection.password;

    if (connection.useWallet) {
        password = null;
    }

    return password;
}

/**
  * @desc - retorna o driver utilizado para o acesso à base de dados
  * @return string
  */
function getDriverClass() {
    var connection = getConnection();
    return connection.driverClass;
}

/**
  * @desc - retorna a tradução da key 
  * @return string
  */
function translate(key, defaultValue){
  var word = reportContext.getMessage(key, reportContext.getLocale());
  return word ? word : defaultValue ? defaultValue : key;
}

/**
  * @desc - retorna o formato de data de acordo com a linguagem
  * @return object
  */
function getDateFormat() {
  var lang = params.__locale.value;
  var dateFormat = "dd/MM/YYYY";
  var formats = {
    "en_us": "MM/dd/YYYY",
  };
  if (lang in formats) dateFormat = formats[lang];
  return dateFormat;
}