<?php
namespace PME\Controllers\Requisicao;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class RequisicaoAnalServController extends \Zeedhi\Framework\Controller\Report {

    protected $entityManager;
    protected $environment;
    protected $birtStrategy;
    protected $utilities;
    protected $lista;
    protected $newCode;

    public function __construct(ReportService $reportService, $entityManager, $environment, $birtStrategy, $utilities, $lista, $newCode) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->birtStrategy  = $birtStrategy;
        $this->utilities     = $utilities;
        $this->lista         = $lista;
        $this->newCode       = $newCode;

        parent::__construct($reportService);
    }

    protected $reportMapping = array (
      'PME00900' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                "CDFILCENTPME"  => "P_CDFILCENTPME",
                "NMFILCENTPME"  => "P_NMFILCENTPME",
                "SERVICO"       => "P_CDSERVFILI",
                "UNIDADE"       => "P_CDFILIAL",
                "CDOPERADOR"    => "P_CDOPERADOR",
                "NMOPERADOR"    => "P_NMOPERADOR",
                "DTDE"          => "P_DTINICIAL",
                "DTATE"         => "P_DTFINAL",
                "NRLISTA"       => "P_NRLISTA",
                "CDPICTPROD"    => "P_CDPICTPROD",
                "TAMNIV"        => "P_TAMNIV",
                "NRLINHABRANCA" => "P_NRLINHABRANCA",
                "QUEBRAPAG"     => "P_IDQUEBRAPAGINA",
                "CDARVPRODINI"  => "P_PRODINI",
                "CDARVPRODFIN"  => "P_PRODFIN",
                "NRORG"         => "P_NRORG"
            )
        ),
      'PME00902' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                "CDFILCENTPME"  => "P_CDFILCENTPME",
                "NMFILCENTPME"  => "P_NMFILCENTPME",
                "DTDE"          => "P_DTINICIAL",
                "DTATE"         => "P_DTFINAL",
                "TAMNIV"        => "P_TAMNIV",
                "CDGRUPFILI"    => "P_CDGRUPFILI",
                "NMGRUPFILI"    => "P_NMGRUPFILI",
                "UNIDADE"       => "P_CDFILIAL",
                "SERVICO"       => "P_CDSERVFILI",
                "CDOPERADOR"    => "P_CDOPERADOR",
                "NMOPERADOR"    => "P_NMOPERADOR",
                "CDPICTPROD"    => "P_CDPICTPROD",
                "NRLISTA"       => "P_NRLISTA",
                "NRLINHABRANCA" => "P_NRLINHABRANCA",
                "CDARVPRODINI"  => "P_PRODINI",
                "CDARVPRODFIN"  => "P_PRODFIN",
                "NRORG"         => "P_NRORG"
            )
        )  
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrorg = $this->environment->getNrOrg();
        
        if(empty($row['CDARVPRODINI'])) $row['CDARVPRODINI'] = 0;
  
        if(empty($row['CDARVPRODFIN'])) $row['CDARVPRODFIN'] = "ZZZZZZZZZZ";
        else $row['CDARVPRODFIN'] = str_pad($row['CDARVPRODFIN'], 10, "Z", STR_PAD_RIGHT);

        if($row['CALCULAR']) $this->procedurePrevCons($row);
        
        $row['CDOPERADOR'] = $this->environment->getCdOperador(); 
        $row['NMOPERADOR'] = $this->environment->getCurrentUserName(); 
        
        if($row['QUEBRAPAG'] === false) $row['QUEBRAPAG'] = 'N';
        else $row['QUEBRAPAG'] = 'S';
        
        if($row['QTDPROD'] === false) $row['QTDPROD'] = 'N';
        else $row['QTDPROD'] = 'S';

        if($row['UNIDADE'] === null){
          $row['UNIDADE'] = 'T';
        }

        if(empty($row['SERVICO'])){
          $row['SERVICO'] = 'T';
        } else if(is_array($row['SERVICO'])){
          $row['SERVICO'] = join("_", $row['SERVICO']);
        }
      
        if($row['TIPOCARD'] === null){
          $tipoCard = join('_', $this->getTipoCard($row));
          $row['NRLISTA'] = $this->lista->geraLista($tipoCard, $nrorg); 
        } else {
          $tipoCard = is_array($row['TIPOCARD']) ? join("_", $row['TIPOCARD']) : $row['TIPOCARD'];
          $row['NRLISTA'] = $this->lista->geraLista($tipoCard, $nrorg); 
        }
        
        if($row['ORDENA'] == 0) $row['TAMNIV'] = 10;
        else $row['TAMNIV'] = 0;
        
        $row['NRORG'] = $nrorg;

        $this->birtStrategy->setReportLocale($this->environment->getLanguage());
        $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));
    }
    
    public function calculatePrevCons(DTO\Request\Row $request, DTO\Response $response) {
      try{
        $row = $request->getRow();
        $nrconsplan = $this->procedurePrevCons($row);
        $response->setMetadata(['nrconsplan' => $nrconsplan]);
      }catch(\Exception $e) {
        $response->setError(new DTO\Response\Error($e->getMessage(), 1));
      }
    }
    
    private function procedurePrevCons($row){
      $nrorg = $this->environment->getNrOrg();
      $row['CDOPERADOR'] = $this->environment->getCdOperador();
      $nrconsplan = $this->newCode->getCodeByTableName('CONSPLAN'.$row['CDFILCENTPME'], $nrorg);

      if(empty($row['UNIDADE'])){
        $cdfilial = join('_', $this->getUnidades($row));
      }else $cdfilial = is_array($row['UNIDADE']) ? join("_", $row['UNIDADE']) : $row['UNIDADE'];
      
      if(empty($row['SERVICO'])){
        $cdservfili = join('_', $this->utilities->getDistinctServicos($row['CDFILCENTPME'], $row['UNIDADE']));
      } else $cdservfili = is_array($row['SERVICO']) ? join("_", $row['SERVICO']) : $row['SERVICO'];
      
      if(empty($row['TIPOCARD'])){
        $tipoCard = join('_', $this->getTipoCard($row));
      }else $tipoCard = is_array($row['TIPOCARD']) ? join("_", $row['TIPOCARD']) : $row['TIPOCARD'];
      
      if($row['QTDPROD'] === false) $row['QTDPROD'] = 'N';
      else $row['QTDPROD'] = 'S';

      $connection = $this->entityManager->getConnection();
      $procedureCons = new StoredProcedure($connection, 'PREVISAO_CONSUMO_PME');
  
      $procedureCons->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $row['CDFILCENTPME'], Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDTIPOCARD',   Param::PARAM_INPUT, $tipoCard,            Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $cdfilial,            Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $cdservfili,          Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTDE'],         Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTATE'],        Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, $row['ACRESCIMO'],    Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $row['CDARVPRODINI'], Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $row['CDARVPRODFIN'], Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'],   Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_DTHRCALCPC',   Param::PARAM_INPUT, date("d/m/Y"),        Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $nrconsplan,          Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, $row['QTDPROD'],      Param::PARAM_TYPE_STR));
      $procedureCons->execute();
      return $nrconsplan;
    }
    
    private function getUnidades($row){
      $rows = $this->utilities->getUnidades($row['CDFILCENTPME']);
      $unidades = [];
      foreach($rows as $unid){
        array_push($unidades, $unid['CDFILIAL']);
      }
      return $unidades;
    }

    private function getTipoCard($row){
      $rows = $this->utilities->getTipoCard($row['CDFILCENTPME']);
      $tiposCard = [];
      foreach($rows as $tipo){
        array_push($tiposCard, $tipo['CDTIPOCARD']);
      }
      return $tiposCard;
    }
}