import {
  Loader, Metadata, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, Select, Button,
} from '@zeedhi/common';

export class Pme01000EmissaoCardSemanaController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');

    if (this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('report/emissaoCardSemana', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  private setParams(row: IDictionary) {
    row.CDTIPOCARD = row.TIPOCARD.CDTIPOCARD;
    row.NMTIPOCARD = row.TIPOCARD.NMTIPOCARD;
    row.NMFILIAGRE = row.UNIDADE ? row.UNIDADE.NMFILIAL : null;
    row.CDFILIAL = row.UNIDADE ? row.UNIDADE.CDFILIAL : this.appController.getCdCentral();
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }

  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      if (this.utilities.dateIsValid(row.DTDE, row.DTATE)) {
        valid = true;
      } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }

  public openModalSearch() {
    this.modalController.openModal();
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD').value;
    this.utilities.setCdTipoCard(tipoCard.CDTIPOCARD);
  }

  public async createPeriodoModal() {
    this.modalController.createModal('/modals/gridPeriodo');
  }

  public destroyPeriodoModal() {
    this.modalController.destroyModal();
  }
}
